"""
Implementation steps:

1. Understand exactly what to implement
   Make decisions on special cases and 
   how to handle them (or handle them at all)
   
   Ex: Return smallest value: what if the
   list is empty? What should I do and what 
   to return?
   
   Ex: Search for the index of a value:
   what if there are many copies, which 
   one to return, first, last, all? What
   if it is not there, return -1 or None?
   
2. To come up with a method to solve
   Write a simple pseudo code
   
3. Come up with test cases to see if it
   works (roughly for these)
   
   Usual test cases: normal lists
   
   Unusual/edge cases: ex: searching in a
   list: values in the beginning or end of
   list
   
4. Think about complexity of solution
    Even small changes can make a big
    difference
    --> never never write n**2 algorithms
    unless it is necessary
    
5. Implement and test (use your own test 
   cases)   
   
Smallest two values problem:

Requirements:

1. We will return the index of the 
   first value found
2. If there is more than one copy of a value, 
   two smallest values can be the same value
3. If the list has less than two values,
   return (None, None)
4. If the does not have numbers, should work
   the same
5. Should the list remain unchanged after
   the function? YES
6. What to return? A tuple
7. Return values in the order they are found
   in the list
   
Possible solutions:

1.
Make a copy of the list
Find the min value and its index
remove it,
Find the min value and its index
If the second min value is after the 
first one, then its index must be adjusted

2. 
Make a copy, sort
Find the two smallest values
Find their index in the original list
If the two values are the same
then the second index must be searched
after the first index

3. 
Go through the list and keep 
track of the smallest two values
and their index seen so far


[2,5,1,6]  #second value is before first
[1,5,2,6]  #second value is after first
[1,5,1,6]  #second value is the same as first
[1,5,6,1]  #values are at the edges of the list
[2,5,6,1]
[5,6,2,1]
[1,2,5,6]


"""


import random
import time

def index_two_v1( L ):
    """
    Complexity: n = len(L)
    O(n) complexity algorithm
    
    >>> index_two_v1([2,5,1,6])
    (2, 0)
    >>> index_two_v1([1,5,2,6])
    (0, 2)
    >>> index_two_v1([1,5,1,6])
    (0, 2)
    >>> index_two_v1([5,6,2,1])
    (3, 2)
    
    """ 
    if len(L) < 2:
        return (None, None)
    
    L2 = L.copy() #n operations: read a value and append
    x = min(L2)   #n comparisons
    i0 = L2.index(x)  #n comparisons
    L2.pop(i0)  #1 comparison operation
    y = min(L2)  #n comparison
    i1 = L2.index(y) #n comparisons
    if i1 >= i0:
        i1 += 1
    return i0, i1


def index_two_v2( L ):
    """
    Complexity: n = len(L)
    O(nlogn + 3n) complexity algorithm
    
    O(nlogn) for simplicity complexity algorithm

    >>> index_two_v2([2,5,1,6])
    (2, 0)
    >>> index_two_v2([1,5,2,6])
    (0, 2)
    >>> index_two_v2([1,5,1,6])
    (0, 2)
    >>> index_two_v2([5,6,2,1])
    (3, 2)
    
    """     
    if len(L) < 2:
        return (None, None)
    
    L2 = L.copy()  #n comparisons
    L2.sort()      #sorting: n*log(n)
    x,y = L2[0], L2[1]
    ## search values in the original list
    i0 = L.index(x)  #n comparison
    if x==y:
        i1 = L.index(y,i0+1)  #n comparisons
    else:
        i1 = L.index(y)
    return i0, i1


def index_two_v3( L ):
    """
    Complexity: O(n)
    (single pass from the list)
    >>> index_two_v3([2,5,1,6])
    (2, 0)
    >>> index_two_v3([1,5,2,6])
    (0, 2)
    >>> index_two_v3([1,5,1,6])
    (0, 2)
    >>> index_two_v3([5,6,2,1])
    (3, 2)
    
    """     
    
    
    if len(L) < 2:
        return (None, None)
    
    if L[0] < L[1]:
        i0,i1 = 0, 1
    else:
        i0, i1 = 1, 0
        
    for i in range(2, len(L)):
        if L[i] < L[i0]:
            i0, i1 = i, i0
        elif L[i] < L[i1]:
            i1 = i
    return i0, i1

def index_two_v4( L ):
    """
    Complexity: O(n)
    >>> index_two_v4([2,5,1,6])
    (2, 0)
    >>> index_two_v4([1,5,2,6])
    (0, 2)
    >>> index_two_v4([1,5,1,6])
    (0, 2)
    >>> index_two_v4([5,6,2,1])
    (3, 2)
    
    """     
    
    
    if len(L) < 2:
        return (None, None)
    
    if L[0] < L[1]:
        i0,i1 = 0, 1
    else:
        i0, i1 = 1, 0
        
    for i in range(2, len(L)):  
        if L[i] < L[i1]:
            if L[i] < L[i0]:
                i0, i1 = i, i0
            else:
                i1 = i
    return i0, i1


def test_func(f,L):
    start = time.time()
    i0,i1 = f(L)
    end = time.time()
    print ("{} returned ({},{}); time {:.3f} seconds".format(f.__name__,i0,i1,end-start))

            


if __name__ == "__main__":
    n = int(input("Enter the number of values to test ==> "))
    values = list(range(0,n))
    random.shuffle( values )

    test_func(index_two_v1, values)
    test_func(index_two_v2, values)
    test_func(index_two_v3, values)
    test_func(index_two_v4, values)
